import unittest, time
from geonames_countries import Countries

class TestCountries(unittest.TestCase):

  def test_findSingularProperties(self):
    self.assertEquals('Australia', Countries.findACountryBytld('.au'))
    self.assertEquals('AU', Countries.findAISOBytld('.au'))
    self.assertEquals('.au', Countries.findAtldByCountry('Australia'))

    self.assertIsNone(Countries.findAtldByCountry('Freedonia'))

  def test_findManyProperties(self):
    countries = Countries.findISOByCurrencyCode('EUR')
    self.assertTrue('IE' in countries)
    self.assertTrue('IT' in countries)
    self.assertFalse('US' in countries) #What a day that'd be

    #Go on break the build ;)
    self.assertTrue('GB' in Countries.findISOByCurrencyCode('GBP'))
    self.assertEquals([], Countries.findISOByCurrencyCode('Freedonia'))

  def test_findDictionaries(self):
    countries = Countries.findByCurrencyCode('EUR')

    #We only get EUR
    self.assertTrue(all([x['CURRENCYCODE'] == 'EUR' for x in countries]))
    self.assertTrue('IT' in [x['ISO'] for x in countries])
    self.assertTrue('IE' in [x['ISO'] for x in countries])
    self.assertEquals([], Countries.findByCurrencyCode('GALLEON'))

if __name__ == "__main__":
  suite = unittest.TestLoader().loadTestsFromTestCase(TestCountries)
  unittest.TextTestRunner(verbosity=2).run(suite)


import json, gzip, os
from collections import defaultdict

class _Countries(object):

  def __init__(self):
    path = os.path.join(os.path.dirname(__file__), 'geonames_countries.json.gz')
    with gzip.open(path, 'r') as f:
      self.country_data = json.load(f)
    if (self.country_data['version'] != 0):
      raise Exception('geonames_countries.Countries got unknown geonames_countries.json.gz version.')
    self.countries = self.country_data['countries']
    self.header = self.country_data['header']

    self._dicts = [dict(zip(self.header, c)) for c in self.countries]
    self._indices = defaultdict(lambda: defaultdict(lambda: []))
    for country in self._dicts:
      for prop, val in country.iteritems():
        self._indices[prop][val].append(country)

  def _data(self):
    return self.country_data

  def dicts(self):
    return self._dicts

  def findXbyY(self, name, mark, delim):
      byIndex = name.index(delim)
      target = name[len(mark):byIndex].upper()
      source = name[byIndex + len(delim):].upper()
      return lambda x: [x[target] for x in self._indices[source][x]]

  def _getFirstOrNone(self, name, x):
    try:
      return self.findXbyY(name, 'findA', 'By')(x)[0]
    except:
      return None

  def __getattr__(self, name):
    if name.startswith('findBy'):
      prop = name[len('findBy'):].upper()
      return lambda x: self._indices[prop][x]
    if name.startswith('findA'):
      name = name
      return lambda x: self._getFirstOrNone(name, x)
    if name.startswith('find'):
      return self.findXbyY(name, 'find', 'By')
    else:
      return self.name
Countries = _Countries()


from distutils.core import setup

setup(
  name = 'geonames_countries',
  packages = ['geonames_countries'],
  version = '0.1.1',
  description = 'A simple wrapper for country code and tld data.',
  author = 'Marco Montagna',
  author_email = 'marcojoemontagna@gmail.com',
  url = 'https://github.com/mmontagna/geonames_countries',
  download_url = 'https://github.com/mmontagna/geonames_countries/archive/v0.1.1.tar.gz',
  keywords = ['tld', 'country code'],
  classifiers = [],
  include_package_data=True,
  package_data={'geonames_countries': ['geonames_countries.json.gz']}
)